package com.ylx.web.controller.massage;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ylx.common.constant.Constants;
import com.ylx.common.core.controller.BaseController;
import com.ylx.common.core.domain.AjaxResult;
import com.ylx.common.core.domain.R;

import com.ylx.common.core.domain.model.WxLoginUser;
import com.ylx.framework.web.service.WxTokenService;
import com.ylx.massage.utils.WxQrCodeUtil;
import com.ylx.massage.utils.WxUtil;
import com.ylx.massage.domain.TWxUser;
import com.ylx.massage.service.TWxUserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author : YJR
 * @className : WxController
 * @description : [描述说明该类的功能]
 * @createTime : 2023/6/27 19:44
 */
@RestController
@Api(tags = {"微信"})
@RequestMapping("/wx")
@Slf4j
public class WxController extends BaseController {
    @Resource
    private WxUtil wxUtil;

    @Resource
    private WxQrCodeUtil wxQrCodeUtil;
    @Resource
    private TWxUserService wxUserService;


    @Resource
    private WxTokenService wxTokenService;

    /**
     * 获取二维码
     *
     * @param param
     * @return
     */
    @ApiOperation("获取二维码")
    @RequestMapping(value = "getwxQrCode", method = RequestMethod.POST)
    public String getWxQrCodeUtil(@RequestBody JSONObject param) {
        String cOpenId = param.getString("cOpenId");
        return wxQrCodeUtil.getWxQrcode(cOpenId);
    }

    /**
     * 小程序登录
     *
     * @param wxLoginUser
     * @return
     */
    @PostMapping("/login")
    @ApiOperation("小程序登录")
    public AjaxResult login(@RequestBody WxLoginUser wxLoginUser) {
        AjaxResult ajax = new AjaxResult();
        try {
            // 验证输入数据的合法性
            if (wxLoginUser == null || wxLoginUser.getCode() == null || wxLoginUser.getCode().isEmpty()) {
                return AjaxResult.error("输入数据不合法");
            }

            //todo 获取用户信息自测方法
            WxLoginUser wxUser = wxUtil.testgetUserOpenId(wxLoginUser.getCode());
            if (wxUser == null) {
                return AjaxResult.error("获取用户信息失败");
            }

            // 生成并返回令牌
            String token = wxTokenService.createToken(wxUser);
            if (token == null || token.isEmpty()) {
                return AjaxResult.error("生成令牌失败");
            }

            ajax.put(Constants.TOKEN, token);
            return ajax;
        } catch (Exception e) {
            // 捕获并处理异常，返回友好的错误信息
            JSONObject errorObj = new JSONObject();
            errorObj.put("message", "操作失败");
            errorObj.put("error", e.getMessage());
            return AjaxResult.error(JSON.toJSONString(errorObj));
        }
    }

    /**
     * 获取电话号
     *
     * @param param
     * @return
     */
    @ApiOperation("获取电话号")
    @RequestMapping(value = "getPhone", method = RequestMethod.POST)
    public R getPhone(@RequestBody JSONObject param) {
        String code = param.getString("code");
        String openId = param.getString("openId");
        if (StringUtils.isBlank(code) || StringUtils.isBlank(openId)) {
            return R.fail("openId或者授权conde为空");
        }
        String phone = wxUtil.getUserPhone(openId, code);
        if (StringUtils.isBlank(phone)) {
            return R.fail("获取手机号信息失败,请重试");
        }
        return R.ok(phone);
    }

    /**
     * 获取用户信息
     *
     * @param
     * @return
     */
    @ApiOperation("获取用户信息")
    @RequestMapping(value = "getUserInfo", method = RequestMethod.GET)
    public R getUserInfo() {
        WxLoginUser wxLoginUser = getWxLoginUser();
        // 检查wxLoginUser是否为null
        if (wxLoginUser == null) {
            return R.fail("用户登录信息不存在");
        }
        LambdaQueryWrapper<TWxUser> queryWrapper = new LambdaQueryWrapper<TWxUser>()
                .eq(TWxUser::getcOpenid, wxLoginUser.getCOpenid());
        TWxUser wxUser;
        try {
            wxUser = wxUserService.getOne(queryWrapper);
        } catch (Exception e) {
            log.error("获取用户信息失败", e);
            return R.fail("获取用户信息失败，系统异常");
        }
        if (wxUser == null) {
            return R.fail("用户信息不存在");
        }
        // 移除用户登录的session key
        wxUser.setcSessionKey(null);
        return R.ok(wxUser);
    }


    @RequestMapping(value = "updateUserInfo", method = RequestMethod.POST)
    public R updateUserInfo(@RequestBody TWxUser wxUser) {
        return R.ok(wxUserService.saveOrUpdate(wxUser));
    }

    /**
     * 更新用户信息
     *
     * @param wxUser
     * @return
     */
    @RequestMapping(value = "updateUserInfoMessage", method = RequestMethod.POST)
    public R updateUserInfoMessage(@RequestBody TWxUser wxUser) {
        QueryWrapper<TWxUser> wrapper = new QueryWrapper<>();
        wrapper.lambda().ne(TWxUser::getcOpenid, wxUser.getcOpenid());
        return R.ok(wxUserService.updateById(wxUser));
    }


    /**
     * 获取用户信息
     *
     * @param param
     * @return
     */
    @RequestMapping(value = "getUser", method = RequestMethod.POST)
    public List<TWxUser> getUser(@RequestBody JSONObject param) {
        QueryWrapper<TWxUser> wrapper = new QueryWrapper<>();
        return wxUserService.list(wrapper);
    }

    @RequestMapping(value = "select", method = RequestMethod.GET)
    public R select(Page<TWxUser> page, TWxUser user) {
        LambdaQueryWrapper<TWxUser> objectLambdaQueryWrapper = new LambdaQueryWrapper<>();
        objectLambdaQueryWrapper.eq(StringUtils.isNotBlank(user.getcOpenid()), TWxUser::getcOpenid, user.getcOpenid()).
                like(StringUtils.isNotBlank(user.getcNickName()), TWxUser::getcNickName, user.getcNickName());
        // 获取查询返回结果
        Page<TWxUser> pageSelect = wxUserService.page(page, objectLambdaQueryWrapper);
        return R.ok(pageSelect);
    }
}
